% Code from John Frain on example from section 2.5 (Frain, 2010)

%% Problem
% This example is based on the macroeconomic system in Example 10.3 of Shone (2002).
% There are 10 equations in this economic model.
% 
% While the aim in Shone (2002) is to examine the system algebraically, here we examine
% it numerically. Often this may be the only way to solve the system and Matlab is a
% suitable tool for this work. The model is too simple to be of any particular use in
% macroeconomics but it does allow one to illustrate the facilities offered by Matlab for
% this kind of work.

%% Initialise and Describe Variables
N = 15 ; % Number of periods for simulation
c = NaN * zeros(N,1); %real consumption
tax = NaN * zeros(N,1); %real tax
yd = NaN * zeros(N,1); %real disposible income
i = NaN * zeros(N,1); % real investment
g = NaN * zeros(N,1); % real government expenditure
e = NaN * zeros(N,1); % real expenditure
y = NaN * zeros(N,1); % real income
md = NaN * zeros(N,1); % real money demand
ms = NaN * zeros(N,1); %real money supply
r = NaN * zeros(N,1); % interest rate

%%Simulate
% g and ms are the policy variables.

t=(1:N)'; % time variable
g = 330 * ones(N,1);
ms = 470 * ones(N,1);
y(1) = 2000;

% The next step is to simulate the model over the required period. In this case this 
% is achieved by a simple reordering of the equations and inverting the money demand
% equation to give an interest rate equation. In the general case we might need a routine
% to solve the set of non linear equations or some routine to maximise a utility function.
% Note that the loop stops one short of the full period and then does the calculations for
% the final period (excluding the income calculation for the period beyond the end of the
% sample under consideration).

for ii = 1:(N-1)
    tax(ii) = -80 + 0.2 * y(ii);
    yd(ii) = y(ii) - tax(ii);
    c(ii) = 110 + 0.75 * yd(ii);
    md(ii) = ms(ii);
    r(ii) = (20 + 0.25* y(ii) -md(ii))/10; % inverting money demand
    i(ii) = 320 -4 * r(ii);
    e(ii) = c(ii) + i(ii) + g(ii);
    y(ii+1) = e(ii);
end

tax(N) = -80 + 0.2 * y(N);
yd(N) = y(N) - tax(N);
c(N) = 110 + 0.75 * yd(N);
md(N) = ms(N);
r(N) = (20 + 0.25* y(N) -md(N))/10;
i(N) = 320 -4 * r(N);
e(N) = c(N) + i(N) + g(N);

% Now output results and save y for later use. note that the system is in equilibrium.
% Note that in printing we use the transpose of base

base = [t,y,yd,c,g-tax,i,r];
fprintf(' t y yd c g-tax i r\n')
fprintf('%7.0f%7.0f%7.0f%7.0f%7.0f%7.0f%7.0f\n',base')
ybase = y;

%% Revised Simulation
% We increase g to 350 and examine the passage to the new equilibrium. Basically we run
% the same program with a different starting value for g.

N = 15 ; % Number of periods for simulation
c = NaN * zeros(N,1); %real consumption
tax = NaN * zeros(N,1); %real tax
yd = NaN * zeros(N,1); %real disposible income
i = NaN * zeros(N,1); % real investment
g = NaN * zeros(N,1); % real government expenditure
e = NaN * zeros(N,1); % real expenditure
y = NaN * zeros(N,1); % real income
md = NaN * zeros(N,1); % real money demand
ms = NaN * zeros(N,1); %real money supply
r = NaN * zeros(N,1); % interest rate

% Policy Variables
g = 350 * ones(N,1);
ms = 470 * ones(N,1);
t=(1:N)';

y(1) = 2000;
for ii = 1:(N-1)
tax(ii) = -80 + 0.2 * y(ii);
yd(ii) = y(ii) - tax(ii);
c(ii) = 110 + 0.75 * yd(ii);
md(ii) = ms(ii);
r(ii) = (20 + 0.25* y(ii) -md(ii))/10; % inverting money demand
i(ii) = 320 -4 * r(ii);
e(ii) = c(ii) + i(ii) + g(ii);
y(ii+1) = e(ii);
end

tax(N) = -80 + 0.2 * y(N);
yd(N) = y(N) - tax(N);
c(N) = 110 + 0.75 * yd(N);
md(N) = ms(N);
r(N) = (20 + 0.25* y(N) -md(N))/10;
i(N) = 320 -4 * r(N);
e(N) = c(N) + i(N) + g(N);

policy = [t,y,yd,c,g-tax,i,r];
fprintf(' t y yd c g-tax i r\n')
fprintf('%7.0f%7.0f%7.0f%7.0f%7.0f%7.0f%7.0f\n',policy')
ypolicy =y;

% Now we compare results in a table and a graph. Note that income converges to a new
% limit.

fprintf(' t ybase ypolicy\n')
fprintf('%7.0f%7.0f%7.0f\n',[t,ybase, ypolicy]')

plot(t,[ybase,ypolicy])
title('Equilibrium and Shocked Macro-system')
xlabel('Period')
ylabel('Income')
axis([1 15 1995 2045])